#!/usr/bin/env node
"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const minimist_1 = __importDefault(require("minimist"));
const cli_1 = require("./cli");
const output_1 = require("./utils/output");
const kleur_1 = require("kleur");
const pkg = require('../package.json');
function program() {
    const _a = minimist_1.default(process.argv.slice(2)), { version, v, help, h } = _a, options = __rest(_a, ["version", "v", "help", "h"]);
    if (version || v) {
        console.log(pkg.version);
        process.exit(0);
    }
    if (help || h) {
        console.log(`

swaggerlint version ${pkg.version}
<${pkg.repository.url.replace('git+', '').replace('.git', '')}>
swaggerlint is a program to lint swagger (OpenAPI v2.0) API specification.

Usage:
    swaggerlint ./path/to/swagger/file.json
    swaggerlint http://url.to/swagger/file.yml

Options:
 -v, --version              print version number
 -h, --help                 show this help
 --config                   add user config (swaggerlint.config.js)

            `.trim());
        process.exit(0);
    }
    cli_1.cli(options).then(({ results, code }) => {
        if (results.length > 1) {
            results.forEach(result => {
                output_1.logErrors(result.errors, result.schema, {
                    filename: result.src,
                    count: false,
                });
            });
            const totalErros = results.reduce((acc, el) => acc + el.errors.length, 0);
            if (totalErros > 0) {
                output_1.logErrorCount(totalErros);
            }
            else {
                console.log(kleur_1.green('No errors found'));
            }
        }
        else {
            const { errors, schema } = results[0];
            if (code === 1) {
                output_1.logErrors(errors, schema, { count: true });
            }
            else {
                console.log(kleur_1.green('No errors found'));
            }
        }
        process.exit(code);
    });
}
program();
