"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cli = void 0;
const swaggerlint_1 = require("./swaggerlint");
const config_1 = require("./utils/config");
const swaggerfile_1 = require("./utils/swaggerfile");
const utils_1 = require("./utils");
const name = 'swaggerlint-core';
function preLintError({ src, msg }) {
    return {
        src,
        errors: [{ name, msg, location: [] }],
        schema: undefined,
    };
}
function cli(opts) {
    return __awaiter(this, void 0, void 0, function* () {
        if (opts._.length === 0) {
            return {
                code: 1,
                results: [
                    preLintError({
                        msg: 'Neither url nor path were provided for your swagger scheme',
                        src: '',
                    }),
                ],
            };
        }
        let schema;
        const configResult = config_1.getConfig(opts.config);
        if (configResult.type === 'fail') {
            return {
                code: 1,
                results: [preLintError({ msg: configResult.error, src: '' })],
            };
        }
        if (configResult.type === 'error') {
            return {
                code: 1,
                results: [
                    preLintError({
                        msg: configResult.error,
                        src: configResult.filepath,
                    }),
                ],
            };
        }
        const { config } = configResult;
        const result = opts._.map((schemaPath) => __awaiter(this, void 0, void 0, function* () {
            /**
             * handling `swagger-lint https://...`
             */
            if (schemaPath.startsWith('http')) {
                const url = schemaPath;
                utils_1.log(`fetching for ${url}`);
                const swaggerFromUrl = yield swaggerfile_1.getSwaggerByUrl(url).catch((e) => {
                    utils_1.log('error fetching url');
                    utils_1.log(e);
                    return null;
                });
                if (swaggerFromUrl === null) {
                    return preLintError({
                        msg: 'Cannot fetch swagger scheme from the provided url',
                        src: url,
                    });
                }
                else {
                    utils_1.log(`got response`);
                    schema = Object.freeze(swaggerFromUrl);
                    const errors = swaggerlint_1.swaggerlint(swaggerFromUrl, config);
                    const res = {
                        src: url,
                        schema,
                        errors,
                    };
                    return res;
                }
            }
            else {
                /**
                 * handling `swagger-lint /path/to/swagger.json`
                 */
                const result = swaggerfile_1.getSwaggerByPath(schemaPath);
                if ('error' in result) {
                    return preLintError({ msg: result.error, src: schemaPath });
                }
                const errors = swaggerlint_1.swaggerlint(result.swagger, config);
                const res = {
                    src: schemaPath,
                    errors,
                    schema: Object.freeze(result.swagger),
                };
                return res;
            }
        }));
        const results = yield Promise.all(result);
        return {
            results,
            code: results.every(x => x.errors.length === 0) ? 0 : 1,
        };
    });
}
exports.cli = cli;
