"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RuleTester = void 0;
const assert = __importStar(require("assert"));
const swaggerlint_1 = require("./swaggerlint");
const tests_1 = require("./utils/tests");
const utils_1 = require("./utils");
class RuleTester {
    constructor(rule) {
        this.rule = rule;
        this.defaultConfig = {
            rules: {
                [rule.name]: true,
            },
        };
    }
    run({ swagger, openapi, }) {
        const { defaultConfig } = this;
        function runTests(sample, name) {
            const makeSchema = name === 'Swagger' ? tests_1.getSwaggerObject : tests_1.getOpenAPIObject;
            describe(name, () => {
                describe('valid', () => {
                    sample.valid.forEach(valid => {
                        it(valid.it, () => {
                            expect(swaggerlint_1.swaggerlint(
                            // @ts-expect-error: mom, typescript is merging arguments again
                            makeSchema(valid.schema), valid.config || defaultConfig)).toEqual([]);
                        });
                    });
                });
                describe('invalid', () => {
                    sample.invalid.forEach(invalid => {
                        it(invalid.it, () => {
                            const actual = swaggerlint_1.swaggerlint(
                            // @ts-expect-error: mom, typescript is merging arguments again
                            makeSchema(invalid.schema), invalid.config || defaultConfig);
                            const expected = invalid.errors;
                            if (actual.length !== expected.length) {
                                assert.fail(`Expected ${expected.length} error${expected.length === 1 ? '' : 's'} but got ${actual.length}.\n\n${JSON.stringify(actual, null, 4)}`);
                            }
                            actual.forEach((actualError, index) => {
                                const expectedError = expected[index];
                                checkForMsgOrMessageId(actualError, index, 'Lint error');
                                checkForMsgOrMessageId(expectedError, index, 'Provided error');
                                if (expectedError.name) {
                                    assert.strictEqual(expectedError.name, actualError.name, `Expected error name "${expectedError.name}" but got "${actualError.name}" at index ${index}.`);
                                }
                                if (expectedError.msg) {
                                    assert.strictEqual(expectedError.msg, actualError.msg, `Expected error message "${expectedError.msg}" but got "${actualError.msg}" at index ${index}.`);
                                }
                                if (utils_1.hasKey('messageId', expectedError) &&
                                    expectedError.messageId) {
                                    const expectedMessageId = expectedError.messageId;
                                    const actualMessageId = utils_1.hasKey('messageId', actualError) &&
                                        actualError.messageId;
                                    assert.strictEqual(expectedMessageId, actualMessageId, `Expected error messageId "${expectedMessageId}" but got "${actualMessageId}" at index ${index}.`);
                                }
                                if (utils_1.hasKey('data', expectedError) &&
                                    expectedError.data) {
                                    assert.deepStrictEqual(expectedError.data, 
                                    // @ts-expect-error: expected since one was provided
                                    actualError.data, 'Expected data for message template does not match actual data at index ${index}.');
                                }
                                if (!utils_1.hasKey('location', expectedError)) {
                                    assert.fail(`Expected error at index ${index} does not have location specified`);
                                }
                                if (utils_1.hasKey('location', expectedError) &&
                                    expectedError.location) {
                                    assert.deepStrictEqual(expectedError.location, actualError.location, 'Expected location does not match with actual error location at index ${index}.');
                                }
                            });
                            expect(actual).toMatchObject(expected);
                        });
                    });
                });
            });
        }
        describe(this.rule.name, () => {
            if (swagger)
                runTests(swagger, 'Swagger');
            if (openapi)
                runTests(openapi, 'OpenAPI');
            if (!swagger && !openapi)
                assert.fail('Neither swagger nor openapi was passed to RuleTester');
        });
    }
}
exports.RuleTester = RuleTester;
function checkForMsgOrMessageId(err, index, errLabel) {
    const hasMsg = typeof err.msg === 'string';
    const hasMessageId = utils_1.hasKey('messageId', err) && typeof err.messageId === 'string';
    if (!hasMsg && !hasMessageId) {
        assert.fail(`${errLabel} with index ${index} expected to have "msg" or "messageId" but has neither`);
    }
}
