"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../../utils");
const name = 'expressive-path-summary';
const messages = {
    noSummary: 'Every path has to have a summary.',
    nonExpressive: `Every path summary should contain at least 2 words. This has "{{summary}}"`,
};
const operationObjectValidator = ({ node, report, location }) => {
    const { summary } = node;
    if (typeof summary === 'string') {
        if (summary.split(' ').length < 2) {
            report({
                messageId: 'nonExpressive',
                data: {
                    summary,
                },
                location: [...location, 'summary'],
            });
        }
    }
    else {
        report({ messageId: 'noSummary' });
    }
};
const rule = utils_1.createRule({
    name,
    docs: {
        recommended: true,
        description: 'Enforces an intentional path summary',
    },
    meta: {
        messages,
    },
    swaggerVisitor: {
        OperationObject: operationObjectValidator,
    },
    openapiVisitor: {
        OperationObject: operationObjectValidator,
    },
});
exports.default = rule;
