"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../../utils");
const name = 'no-ref-properties';
const rule = utils_1.createRule({
    name,
    docs: {
        recommended: true,
        description: 'Disallows to have additional properties in Reference objects',
    },
    meta: {
        messages: {
            noRefProps: 'To add properties to Reference Object, wrap it in `allOf` and add the properties to the added sibling SchemaObject',
        },
    },
    swaggerVisitor: {
        ReferenceObject: ({ node, report }) => {
            const addonProps = Object.keys(node).filter(x => x !== '$ref');
            if (addonProps.length) {
                report({ messageId: 'noRefProps' });
            }
        },
    },
    openapiVisitor: {
        ReferenceObject: ({ node, report }) => {
            const addonProps = Object.keys(node).filter(x => x !== '$ref');
            if (addonProps.length) {
                report({ messageId: 'noRefProps' });
            }
        },
    },
});
exports.default = rule;
