"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../../utils");
const name = 'no-single-allof';
const rule = utils_1.createRule({
    name,
    docs: {
        recommended: true,
        description: 'Object types should not have a redundant single `allOf` property',
    },
    meta: {
        messages: {
            msg: 'Redundant use of "allOf" with a single item in it.',
        },
    },
    swaggerVisitor: {
        SchemaObject: ({ node, report, location }) => {
            if ('allOf' in node && node.allOf.length === 1) {
                report({ messageId: 'msg', location: [...location, 'allOf'] });
            }
        },
    },
    openapiVisitor: {
        SchemaObject: ({ node, report, location }) => {
            if (node.allOf && node.allOf.length === 1) {
                report({ messageId: 'msg', location: [...location, 'allOf'] });
            }
        },
    },
});
exports.default = rule;
