"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const case_1 = __importDefault(require("case"));
const utils_1 = require("../../utils");
const utils_2 = require("../../utils");
const name = 'object-prop-casing';
const rule = utils_1.createRule({
    name,
    docs: {
        recommended: true,
        description: 'Casing for your object property names',
    },
    meta: {
        messages: {
            casing: 'Property "{{propName}}" has wrong casing. Should be "{{correctVersion}}".',
        },
        schema: {
            type: 'array',
            items: [
                {
                    type: 'string',
                    enum: Object.keys(utils_2.validCases),
                },
                {
                    type: 'object',
                    required: ['ignore'],
                    properties: {
                        ignore: {
                            type: 'array',
                            items: {
                                type: 'string',
                            },
                        },
                    },
                },
            ],
            minItems: 1,
            maxItems: 2,
        },
    },
    swaggerVisitor: {
        SchemaObject: ({ node, report, setting, location }) => {
            if (typeof setting === 'boolean')
                return;
            const [settingCasingName, opts = {}] = setting;
            const IGNORE_PROPERTIES = new Set(Array.isArray(opts.ignore) ? opts.ignore : []);
            if (typeof settingCasingName === 'string' &&
                utils_2.isValidCaseName(settingCasingName)) {
                const validPropCases = utils_2.validCases[settingCasingName];
                if ('properties' in node && node.properties) {
                    Object.keys(node.properties).forEach(propName => {
                        if (IGNORE_PROPERTIES.has(propName))
                            return;
                        const propCase = case_1.default.of(propName);
                        if (!validPropCases.has(propCase)) {
                            const correctVersion = case_1.default[settingCasingName](propName);
                            report({
                                messageId: 'casing',
                                data: {
                                    propName,
                                    correctVersion,
                                },
                                location: [...location, 'properties', propName],
                            });
                        }
                    });
                }
                return;
            }
        },
    },
    openapiVisitor: {
        SchemaObject: ({ node, report, location, setting }) => {
            if (typeof setting === 'boolean')
                return;
            const [settingCasingName, opts = {}] = setting;
            const IGNORE_PROPERTIES = new Set(Array.isArray(opts.ignore) ? opts.ignore : []);
            if (!(typeof settingCasingName === 'string' &&
                utils_2.isValidCaseName(settingCasingName)))
                return;
            const validPropCases = utils_2.validCases[settingCasingName];
            if ('properties' in node &&
                node.properties &&
                typeof node.properties === 'object') {
                Object.keys(node.properties).forEach(propName => {
                    if (IGNORE_PROPERTIES.has(propName))
                        return;
                    const propCase = case_1.default.of(propName);
                    if (!validPropCases.has(propCase)) {
                        const correctVersion = case_1.default[settingCasingName](propName);
                        report({
                            messageId: 'casing',
                            data: {
                                propName,
                                correctVersion,
                            },
                            location: [...location, 'properties', propName],
                        });
                    }
                });
            }
        },
    },
    defaultSetting: ['camel'],
});
exports.default = rule;
