"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../../utils");
const mime_db_1 = __importDefault(require("mime-db"));
const name = 'only-valid-mime-types';
const messages = {
    invalid: '"{{mimeType}}" is not a valid mime type.',
};
function isValidMimeType(maybeMime) {
    return maybeMime in mime_db_1.default;
}
function onlyValidMimeTypeCheck({ node, report, location }) {
    const { consumes, produces } = node;
    if (consumes) {
        consumes.forEach((mType, i) => {
            if (!isValidMimeType(mType)) {
                report({
                    messageId: 'invalid',
                    data: {
                        mimeType: mType,
                    },
                    location: [...location, 'consumes', String(i)],
                });
            }
        });
    }
    if (produces) {
        produces.forEach((mType, i) => {
            if (!isValidMimeType(mType)) {
                report({
                    messageId: 'invalid',
                    data: {
                        mimeType: mType,
                    },
                    location: [...location, 'produces', String(i)],
                });
            }
        });
    }
}
const rule = utils_1.createRule({
    name,
    docs: {
        recommended: true,
        description: 'Checks mime types against known from [`mime-db`](https://npm.im/mime-db)',
    },
    meta: { messages },
    swaggerVisitor: {
        SwaggerObject: onlyValidMimeTypeCheck,
        OperationObject: onlyValidMimeTypeCheck,
    },
    openapiVisitor: {
        MediaTypeObject: ({ location, report }) => {
            const mimeType = location[location.length - 1];
            if (!isValidMimeType(mimeType)) {
                report({
                    messageId: 'invalid',
                    data: {
                        mimeType,
                    },
                });
            }
        },
    },
});
exports.default = rule;
