"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../../utils");
const name = 'required-operation-tags';
const messages = {
    missingTags: 'Operation "{{method}}" in "{{url}}" is missing tags.',
};
function OperationObject({ node, report, location }) {
    if (!Array.isArray(node.tags) || node.tags.length < 1) {
        const method = location[location.length - 1];
        const url = location[location.length - 2];
        report({
            messageId: 'missingTags',
            data: {
                method,
                url,
            },
        });
    }
}
const rule = utils_1.createRule({
    name,
    docs: {
        recommended: true,
        description: 'All operations must have tags',
    },
    meta: {
        messages,
    },
    swaggerVisitor: {
        OperationObject,
    },
    openapiVisitor: {
        OperationObject,
    },
});
exports.default = rule;
