"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../../utils");
const name = 'required-parameter-description';
const messages = {
    missingDesc: '"{{name}}" parameter is missing description.',
};
function ParameterObject({ node, report, location }) {
    if (!('description' in node)) {
        report({
            messageId: 'missingDesc',
            data: {
                name: node.name,
            },
        });
    }
    else if (typeof node.description === 'string' && !node.description) {
        report({
            messageId: 'missingDesc',
            data: {
                name: node.name,
            },
            location: [...location, 'description'],
        });
    }
}
const rule = utils_1.createRule({
    name,
    docs: {
        recommended: true,
        description: 'All parameters must have description',
    },
    meta: {
        messages,
    },
    swaggerVisitor: {
        ParameterObject,
    },
    openapiVisitor: {
        ParameterObject,
    },
});
exports.default = rule;
