"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../../utils");
const name = 'required-tag-description';
const messages = {
    missingDesc: 'Tag "{{name}}" is missing description.',
};
function TagObject({ node, report, location }) {
    if (!('description' in node)) {
        report({
            messageId: 'missingDesc',
            data: {
                name: node.name,
            },
        });
        return;
    }
    if (!node.description) {
        report({
            messageId: 'missingDesc',
            data: {
                name: node.name,
            },
            location: [...location, 'description'],
        });
    }
}
const rule = utils_1.createRule({
    name,
    docs: {
        recommended: true,
        description: 'All tags must have description',
    },
    meta: {
        messages,
    },
    swaggerVisitor: {
        TagObject,
    },
    openapiVisitor: {
        TagObject,
    },
});
exports.default = rule;
