"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.swaggerlint = void 0;
const utils_1 = require("./utils");
const validate_json_1 = require("./utils/validate-json");
const swagger_1 = require("./utils/swagger");
const oaUtils = __importStar(require("./utils/openapi"));
const defaultConfig_1 = __importDefault(require("./defaultConfig"));
const rules_1 = require("./rules");
const walker = __importStar(require("./walker"));
function validateInput(schema) {
    if (swagger_1.isSwaggerObject(schema)) {
        return {
            _type: 'swagger',
            schema,
        };
    }
    if (oaUtils.isValidOpenAPIObject(schema)) {
        return {
            _type: 'openAPI',
            schema,
        };
    }
    return null;
}
function swaggerlint(input, config) {
    const errors = [];
    const validated = validateInput(input);
    if (validated === null) {
        return [
            {
                msg: 'You have supplier neither OpenAPI nor Swagger schema',
                name: 'swaggerlint-core',
                location: [],
            },
        ];
    }
    const walkerResult = validated._type === 'swagger'
        ? walker.walkSwagger(validated.schema, config.ignore)
        : walker.walkOpenAPI(validated.schema, config.ignore || {});
    if ('errors' in walkerResult) {
        return walkerResult.errors;
    }
    const info = Object.assign(Object.assign({}, validated), { visitors: walkerResult.visitors });
    const validatedRules = Object.keys(config.rules).reduce((acc, ruleName) => {
        var _a;
        const rule = rules_1.rules[ruleName];
        if (!rule) {
            errors.push({
                msg: `swaggerlint.config.js contains unknown rule "${ruleName}"`,
                name: 'swaggerlint-core',
                location: [],
            });
            return acc;
        }
        let setting = config.rules[ruleName];
        if (setting === false) {
            return acc;
        }
        if (setting === true) {
            const defaultConfigSetting = defaultConfig_1.default.rules[ruleName];
            if (typeof defaultConfigSetting !== 'undefined') {
                setting = defaultConfigSetting;
            }
            else if ('defaultSetting' in rule) {
                setting = rule.defaultSetting;
            }
        }
        if (rule.meta && 'schema' in rule.meta && ((_a = rule.meta) === null || _a === void 0 ? void 0 : _a.schema)) {
            const ruleSettingErrors = validate_json_1.validate(rule.meta.schema, setting);
            if (ruleSettingErrors.length) {
                ruleSettingErrors.forEach(se => {
                    const err = {
                        name: rule.name,
                        msg: 'Invalid rule setting.',
                        location: [],
                    };
                    switch (se.keyword) {
                        case 'enum':
                            if (typeof se.schema[0] === 'string') {
                                const gotValue = JSON.stringify(se.data);
                                const expectedValues = se.schema
                                    .map((x) => `"${x}"`)
                                    .join(', ');
                                err.msg = `Invalid rule setting. Got ${gotValue}, expected: ${expectedValues}`;
                            }
                            break;
                        case 'additionalProperties':
                            if (utils_1.hasKey('additionalProperty', se.params)) {
                                err.msg = `Unexpected property "${se.params.additionalProperty}" in rule settings`;
                            }
                            break;
                        case 'maxItems':
                            err.msg = `Rule setting ${se.message}`;
                            break;
                        case 'minItems':
                            err.msg = `Rule setting ${se.message}`;
                            break;
                        case 'required':
                            err.msg = `${utils_1.capitalize(se.message || 'Missing required field')}, got ${typeof se.data}`;
                            break;
                        case 'type':
                            err.msg = `Value ${se.message}, got ${typeof se.data}`;
                            break;
                    }
                    errors.push(err);
                });
                return acc;
            }
        }
        acc.push({
            rule,
            setting,
        });
        return acc;
    }, []);
    if (validatedRules.length === 0) {
        errors.push({
            msg: 'Found 0 enabled rules. Swaggerlint requires at least one rule enabled.',
            name: 'swaggerlint-core',
            location: [],
        });
        return errors;
    }
    if (info._type === 'swagger') {
        validatedRules.forEach(({ rule, setting }) => {
            const { swaggerVisitor } = rule;
            if (!swaggerVisitor)
                return;
            Object.keys(swaggerVisitor).forEach(visitorName => {
                // TODO swagger & openapi visitor names checks
                if (!utils_1.isValidSwaggerVisitorName(visitorName))
                    return;
                const check = swaggerVisitor[visitorName];
                const specificVisitor = info.visitors[visitorName];
                specificVisitor.forEach(
                /**
                 * TODO: note the type for `node`
                 * ts infers example object yet it can be any of the objects
                 */
                ({ node, location }) => {
                    const report = makeReportFunc(errors, rule, location);
                    if (typeof check === 'function') {
                        /**
                         * ts manages to only infer example object here,
                         * due to the checks above function call is supposed to be safe
                         *
                         */
                        // @ts-expect-error: @see https://bit.ly/2MNEii7
                        check({ node, location, setting, report, config });
                    }
                });
            });
        });
    }
    else {
        validatedRules.forEach(({ rule, setting }) => {
            const { openapiVisitor } = rule;
            if (!openapiVisitor)
                return;
            Object.keys(openapiVisitor).forEach(visitorName => {
                if (!oaUtils.isValidVisitorName(visitorName))
                    return;
                const check = openapiVisitor[visitorName];
                if (check === undefined)
                    return;
                const specificVisitor = info.visitors[visitorName];
                specificVisitor.forEach(({ node, location }) => {
                    const report = makeReportFunc(errors, rule, location);
                    check({ node, location, setting, report, config });
                });
            });
        });
    }
    return errors;
}
exports.swaggerlint = swaggerlint;
function makeReportFunc(errors, rule, location) {
    return function (arg) {
        var _a, _b;
        if (utils_1.hasKey('messageId', arg) && arg.messageId) {
            const msgTemplate = ((_b = (_a = rule.meta) === null || _a === void 0 ? void 0 : _a.messages) === null || _b === void 0 ? void 0 : _b[arg.messageId]) || '';
            // TODO: return an error for the rule if `messageId` is unknown
            /* eslint-disable indent */
            const message = arg.data
                ? Object.keys(arg.data).reduce((acc, key) => {
                    return acc.replace(new RegExp(`{{\s*${key}\s*}}`, 'g'), 
                    // @ts-expect-error: this code will not run if data is undefined.
                    arg.data[key]);
                }, msgTemplate)
                : msgTemplate;
            /* eslint-enable indent */
            const err = {
                name: rule.name,
                msg: message,
                messageId: arg.messageId,
                location: arg.location || location,
            };
            if (arg.data)
                err.data = arg.data;
            errors.push(err);
            return;
        }
        if (utils_1.hasKey('message', arg) && arg.message) {
            const err = {
                name: rule.name,
                msg: arg.message,
                location: arg.location || location,
            };
            errors.push(err);
            return;
        }
        throw new Error(`Invalid data passed to report function:\n\n${JSON.stringify(arg, null, 4)}`);
    };
}
