"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfig = exports.resolveConfigExtends = void 0;
const path_1 = __importDefault(require("path"));
const cosmiconfig_1 = require("cosmiconfig");
const validate_json_1 = require("./validate-json");
const defaultConfig_1 = __importDefault(require("../defaultConfig"));
const pkg = require('../../package.json');
function mergeConfigs(defConf, userConf) {
    var _a, _b;
    return {
        extends: [...((_a = defConf.extends) !== null && _a !== void 0 ? _a : []), ...((_b = userConf.extends) !== null && _b !== void 0 ? _b : [])],
        rules: Object.assign(Object.assign({}, defConf.rules), userConf.rules),
        ignore: Object.assign({}, userConf.ignore),
    };
}
// workaround for extends keyword
function omitExtends(_a) {
    var { 
    /* eslint-disable @typescript-eslint/no-unused-vars */
    extends: ext } = _a, rest = __rest(_a, ["extends"]);
    return rest;
}
function resolveConfigExtends(baseConfig) {
    const processedConfigs = new Set([]);
    const toBeMergedConfigs = [omitExtends(baseConfig)];
    function resolveExtends(conf) {
        [...(conf.extends || [])]
            .reverse()
            .forEach(function resolveEachName(name) {
            let resolvedName;
            try {
                resolvedName = require.resolve(name);
            }
            catch (e) {
                if (e.code && e.code === 'MODULE_NOT_FOUND') {
                    throw `"${name}" in extends of your config cannot be found. Make sure it exists in your node_modules.`;
                }
                else {
                    throw e;
                }
            }
            if (processedConfigs.has(resolvedName))
                return;
            processedConfigs.add(resolvedName);
            const nextConf = require(name);
            if (nextConf.extends)
                resolveExtends(nextConf);
            toBeMergedConfigs.unshift(nextConf);
        });
    }
    resolveExtends(baseConfig);
    /**
     * default config is always the base of any config
     */
    const result = toBeMergedConfigs.reduce((acc, conf) => mergeConfigs(acc, conf), defaultConfig_1.default);
    return result;
}
exports.resolveConfigExtends = resolveConfigExtends;
const arrayOfStrings = { type: 'array', items: { type: 'string' } };
const configSchema = {
    type: 'object',
    properties: {
        rules: {
            type: 'object',
        },
        extends: arrayOfStrings,
        ignore: {
            type: 'object',
            properties: {
                definitions: arrayOfStrings,
                components: {
                    type: 'object',
                    properties: {
                        schemas: arrayOfStrings,
                        responses: arrayOfStrings,
                        parameters: arrayOfStrings,
                        examples: arrayOfStrings,
                        requestBodies: arrayOfStrings,
                        headers: arrayOfStrings,
                        securitySchemes: arrayOfStrings,
                        links: arrayOfStrings,
                        callbacks: arrayOfStrings,
                    },
                    additionalProperties: false,
                },
            },
            additionalProperties: false,
        },
    },
    additionalProperties: false,
};
const validateConfig = (config) => {
    const errors = validate_json_1.validate(configSchema, config);
    return errors.map(se => {
        const result = {
            name: 'swaggerlint-core',
            msg: 'Invalid config',
            location: [],
        };
        switch (se.keyword) {
            case 'additionalProperties':
                const key = 'additionalProperty' in se.params &&
                    se.params.additionalProperty;
                result.msg = `Unexpected property ${JSON.stringify(key || '')} in swaggerlint.config.js`;
                break;
            case 'type':
                result.msg = `${(se.message || '').replace('should be', 'Expected')} at "${se.dataPath}", got \`${se.data}\``;
        }
        return result;
    });
};
const defaultConfigPath = path_1.default.join(__dirname, '..', 'defaultConfig.js');
function getConfig(configPath) {
    const cosmiconfig = cosmiconfig_1.cosmiconfigSync(pkg.name);
    if (configPath) {
        const cosmiResult = cosmiconfig.load(path_1.default.resolve(configPath));
        if (cosmiResult !== null) {
            const validationErrros = validateConfig(cosmiResult.config);
            if (validationErrros.length) {
                return {
                    type: 'error',
                    filepath: cosmiResult.filepath,
                    error: validationErrros[0].msg,
                };
            }
            const config = cosmiResult.config.extends
                ? resolveConfigExtends(cosmiResult.config)
                : mergeConfigs(defaultConfig_1.default, cosmiResult.config);
            return {
                type: 'success',
                config,
                filepath: cosmiResult.filepath,
            };
        }
        else {
            return {
                type: 'fail',
                error: 'Swaggerlint config with a provided path does not exits.',
            };
        }
    }
    else {
        const cosmiResult = cosmiconfig.search();
        if (cosmiResult !== null) {
            const validationErrros = validateConfig(cosmiResult.config);
            if (validationErrros.length) {
                return {
                    type: 'error',
                    filepath: cosmiResult.filepath,
                    error: validationErrros[0].msg,
                };
            }
            if (cosmiResult.config.extends) {
                try {
                    return {
                        type: 'success',
                        config: resolveConfigExtends(cosmiResult.config),
                        filepath: cosmiResult.filepath,
                    };
                }
                catch (e) {
                    return {
                        type: 'error',
                        error: e,
                        filepath: cosmiResult.filepath,
                    };
                }
            }
            else {
                return {
                    type: 'success',
                    config: mergeConfigs(defaultConfig_1.default, cosmiResult.config),
                    filepath: cosmiResult.filepath,
                };
            }
        }
        else {
            /**
             * if no config is found we use default config
             * reasoning: cli should work out of the box
             */
            return {
                type: 'success',
                config: defaultConfig_1.default,
                filepath: defaultConfigPath,
            };
        }
    }
}
exports.getConfig = getConfig;
