"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.logErrors = exports.logErrorCount = void 0;
const kleur_1 = require("kleur");
const PAD = 6;
function shallowStringify(schema, location) {
    let topLevelObject = true;
    const target = location.reduce(
    // @ts-expect-error: acc is any and it is okay
    (acc, key) => acc === null || acc === void 0 ? void 0 : acc[key], schema);
    const stringifiedObj = JSON.stringify(target, (_, value) => {
        if (Array.isArray(value)) {
            return `Array(${value.length})`;
        }
        if (typeof value === 'object' && value !== null) {
            return topLevelObject
                ? ((topLevelObject = false), value)
                : 'Object';
        }
        return value;
    }, PAD + 2);
    return kleur_1.grey((stringifiedObj || '')
        .split('\n')
        .map((x) => x.padStart(PAD + 1, ' '))
        .join('\n'));
}
function toOneLinerFormat({ msg, name, location }, swagger) {
    const hasLocation = location.length;
    const locationInfo = hasLocation ? ` in ${location.join('.')}` : '';
    const ruleName = kleur_1.dim(name);
    return [
        `${kleur_1.red('error')}${locationInfo}`,
        ruleName.padStart(ruleName.length + PAD, ' '),
        msg.padStart(msg.length + PAD, ' '),
        hasLocation && swagger && shallowStringify(swagger, location),
    ]
        .filter(Boolean)
        .join('\n');
}
function logErrorCount(count) {
    console.log(kleur_1.bold(`You have ${count} error${!!count ? 's' : ''}.`));
}
exports.logErrorCount = logErrorCount;
function logErrors(errors, schema, options = {}) {
    if (options.filename)
        console.log(`\n${kleur_1.bold(options.filename)}`);
    console.log(errors.map(x => toOneLinerFormat(x, schema)).join('\n'));
    if (options.count)
        logErrorCount(errors.length);
}
exports.logErrors = logErrors;
