"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSwaggerByUrl = exports.getSwaggerByPath = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const node_fetch_1 = __importDefault(require("node-fetch"));
function isYamlPath(p) {
    const ext = path_1.default.extname(p);
    return ext === '.yml' || ext === '.yaml';
}
function getSwaggerByPath(pth) {
    const swaggerPath = path_1.default.resolve(pth);
    // non existing path
    if (!fs_1.default.existsSync(swaggerPath)) {
        return {
            error: 'File at the provided path does not exist.',
            swagger: null,
        };
    }
    const isYaml = isYamlPath(swaggerPath);
    try {
        const swagger = isYaml
            ? js_yaml_1.default.safeLoad(fs_1.default.readFileSync(swaggerPath, 'utf8'))
            : require(swaggerPath);
        return {
            swagger,
        };
    }
    catch (e) {
        return {
            swagger: null,
            error: 'Error requiring swaggerlint config',
        };
    }
}
exports.getSwaggerByPath = getSwaggerByPath;
function getSwaggerByUrl(url) {
    return __awaiter(this, void 0, void 0, function* () {
        return node_fetch_1.default(url).then(x => isYamlPath(url) ? x.text().then(js_yaml_1.default.safeLoad) : x.json());
    });
}
exports.getSwaggerByUrl = getSwaggerByUrl;
